@@ -94,7 +94,7 @@ typedef enum readstat_error_e {
     READSTAT_ERROR_NAME_CONTAINS_ILLEGAL_CHARACTER,
     READSTAT_ERROR_NAME_IS_RESERVED_WORD,
     READSTAT_ERROR_NAME_IS_TOO_LONG,
-    READSTAT_ERROR_BAD_TIMESTAMP,
+    READSTAT_ERROR_BAD_TIMESTAMP_STRING,
     READSTAT_ERROR_BAD_FREQUENCY_WEIGHT,
     READSTAT_ERROR_TOO_MANY_MISSING_VALUE_DEFINITIONS,
     READSTAT_ERROR_NOTE_IS_TOO_LONG,
@@ -103,7 +103,8 @@ typedef enum readstat_error_e {
     READSTAT_ERROR_ROW_IS_TOO_WIDE_FOR_PAGE,
     READSTAT_ERROR_TOO_FEW_COLUMNS,
     READSTAT_ERROR_TOO_MANY_COLUMNS,
-    READSTAT_ERROR_NAME_IS_ZERO_LENGTH
+    READSTAT_ERROR_NAME_IS_ZERO_LENGTH,
+    READSTAT_ERROR_BAD_TIMESTAMP_VALUE
 } readstat_error_t;
 
 const char *readstat_error_message(readstat_error_t error_code);
@@ -294,7 +295,11 @@ typedef int (*readstat_value_label_handler)(const char *val_labels,
 typedef void (*readstat_error_handler)(const char *error_message, void *ctx);
 typedef int (*readstat_progress_handler)(double progress, void *ctx);
 
-#if defined _WIN32 || defined __CYGWIN__
+#if defined(_MSC_VER)
+#include <BaseTsd.h>
+typedef SSIZE_T ssize_t;
+typedef _off_t readstat_off_t;
+#elif defined _WIN32 || defined __CYGWIN__
 typedef _off64_t readstat_off_t;
 #elif defined _AIX
 typedef off64_t readstat_off_t;
@@ -341,6 +346,7 @@ typedef struct readstat_parser_s {
     const char             *input_encoding;
     const char             *output_encoding;
     long                    row_limit;
+    long                    row_offset;
 } readstat_parser_t;
 
 readstat_parser_t *readstat_parser_init(void);
@@ -372,6 +378,7 @@ readstat_error_t readstat_set_file_character_encoding(readstat_parser_t *parser,
 readstat_error_t readstat_set_handler_character_encoding(readstat_parser_t *parser, const char *encoding);
 
 readstat_error_t readstat_set_row_limit(readstat_parser_t *parser, long row_limit);
+readstat_error_t readstat_set_row_offset(readstat_parser_t *parser, long row_offset);
 
 /* Parse binary / portable files */
 readstat_error_t readstat_parse_dta(readstat_parser_t *parser, const char *path, void *user_ctx);
@@ -476,7 +483,7 @@ typedef struct readstat_writer_s {
 
     int                         row_count;
     int                         current_row;
-    char                        file_label[100];
+    char                        file_label[257];
     char                        table_name[33];
     const readstat_variable_t  *fweight_variable;
 
