@@ -658,6 +658,13 @@ static readstat_error_t dta_handle_rows(dta_ctx_t *ctx) {
         goto cleanup;
     }
 
+    if (ctx->row_offset) {
+        if (io->seek(ctx->record_len * ctx->row_offset, READSTAT_SEEK_CUR, io->io_ctx) == -1) {
+            retval = READSTAT_ERROR_SEEK;
+            goto cleanup;
+        }
+    }
+
     for (i=0; i<ctx->row_limit; i++) {
         if (io->read(buf, ctx->record_len, io->io_ctx) != ctx->record_len) {
             retval = READSTAT_ERROR_READ;
@@ -672,8 +679,8 @@ static readstat_error_t dta_handle_rows(dta_ctx_t *ctx) {
         }
     }
 
-    if (ctx->row_limit < ctx->nobs) {
-        if (io->seek(ctx->record_len * (ctx->nobs - ctx->row_limit), READSTAT_SEEK_CUR, io->io_ctx) == -1)
+    if (ctx->row_limit < ctx->nobs - ctx->row_offset) {
+        if (io->seek(ctx->record_len * (ctx->nobs - ctx->row_offset - ctx->row_limit), READSTAT_SEEK_CUR, io->io_ctx) == -1)
             retval = READSTAT_ERROR_SEEK;
     }
 
@@ -897,12 +904,10 @@ static readstat_error_t dta_read_label_and_timestamp(dta_ctx_t *ctx) {
                 memmove(timestamp_buffer+1, timestamp_buffer, timestamp_len-1);
                 timestamp_buffer[0] = last_data_label_char;
             }
-            if ((retval = dta_parse_timestamp(timestamp_buffer, timestamp_len,
-                            &timestamp, ctx->handle.error, ctx->user_ctx)) != READSTAT_OK) {
-                goto cleanup;
+            if (dta_parse_timestamp(timestamp_buffer, timestamp_len,
+                        &timestamp, ctx->handle.error, ctx->user_ctx) == READSTAT_OK) {
+                ctx->timestamp = mktime(&timestamp);
             }
-
-            ctx->timestamp = mktime(&timestamp);
         }
     }
 
@@ -1048,7 +1053,7 @@ static readstat_error_t dta_handle_value_labels(dta_ctx_t *ctx) {
                     goto cleanup;
                 }
             }
-        } else if (len > 8) {
+        } else if (len >= 8) {
             if ((retval = dta_read_tag(ctx, "</lbl>")) != READSTAT_OK) {
                 goto cleanup;
             }
@@ -1178,8 +1183,15 @@ readstat_error_t readstat_parse_dta(readstat_parser_t *parser, const char *path,
     ctx->user_ctx = user_ctx;
     ctx->file_size = file_size;
     ctx->handle = parser->handlers;
-    ctx->row_limit = ctx->nobs;
-    if (parser->row_limit > 0 && parser->row_limit < ctx->nobs)
+    if (parser->row_offset > 0)
+        ctx->row_offset = parser->row_offset;
+    int64_t nobs_after_skipping = ctx->nobs - ctx->row_offset;
+    if (nobs_after_skipping < 0) {
+        nobs_after_skipping = 0;
+        ctx->row_offset = ctx->nobs;
+    }
+    ctx->row_limit = nobs_after_skipping;
+    if (parser->row_limit > 0 && parser->row_limit < nobs_after_skipping)
         ctx->row_limit = parser->row_limit;
 
     retval = dta_update_progress(ctx);
